==> audiochain-1.1.5-src.tgz: org/audiochain/ui/gui/mixer/GuiAudioRecorder.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static java.awt.KeyboardFocusManager.*;
import static org.audiochain.ui.gui.BackgroundImageComponent.*;
import static org.audiochain.ui.gui.GlobalGuiContext.*;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;

import org.audiochain.model.AudioMixer;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioProjectLifecycleListener;
import org.audiochain.ui.AudioProjectModificationObserver;
import org.audiochain.ui.AudioProjectModificationObserverListener;
import org.audiochain.ui.gui.BackgroundImageComponent;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.GuiSettings;
import org.audiochain.ui.sync.SyncManager;


/**
 * This is the GUI main class.
 */
public class GuiAudioRecorder extends JFrame implements AudioProjectLifecycleListener {
	private static final long serialVersionUID = 1L;
	private static final String brandName = "AudioChain"; // ✇ is unicode decimal 9991, hex 2797
	private Map<AudioProject, AudioProjectComponent> audioProjectComponents = new LinkedHashMap<AudioProject, AudioProjectComponent>();
	private JComponent backgroundImageComponent;
	private final AudioProjectLifecycleManager audioProjectLifecycleManager;
	private final KeyEventManager keyEventManager;

	public GuiAudioRecorder() {
		audioProjectLifecycleManager = new AudioProjectLifecycleManager();
		audioProjectLifecycleManager.addAudioProjectLifecycleListener(this);
		keyEventManager = new KeyEventManager(this);
		init();
	}

	private void init() {
		final GuiSettings settings = getGuiSettings();
		settings.initializeUIManager();
		getCurrentKeyboardFocusManager().addKeyEventDispatcher(keyEventManager);
		setJMenuBar(new GuiAudioRecorderMenu(this));
		backgroundImageComponent = new BackgroundImageComponent("/org/audiochain/ui/gui/background.png");
		backgroundImageComponent.setLayout(new BorderLayout());
		setContentPane(backgroundImageComponent);
		setBounds(settings.getWindowBounds());
		updateTitle();
		setVisible(true);
		setExtendedState(settings.getWindowExtendedState());
		ImageIcon icon = readIcon("/org/audiochain/ui/gui/mixer/frameicon.png");
		if (icon != null) {
			setIconImage(icon.getImage());
		}
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				if (getExtendedState() == NORMAL) {
					settings.setWindowWidth(getWidth());
					settings.setWindowHeight(getHeight());
				}
			}

			@Override
			public void componentMoved(ComponentEvent e) {
				if (getExtendedState() == NORMAL) {
					settings.setWindowLocationX(getX());
					settings.setWindowLocationY(getY());
				}
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				exit();
			}
		});
		addWindowStateListener(new WindowStateListener() {
			@Override
			public void windowStateChanged(WindowEvent e) {
				settings.setWindowExtendedState(getExtendedState());
			}
		});
		final SyncManager syncManager = SyncManager.getInstance();
		syncManager.setAudioProjectLifecycleObservable(audioProjectLifecycleManager);

		new Thread() {
			@Override
			public void run() {
				setPriority(MIN_PRIORITY);
				AudioMixer.getInstance();
				syncManager.addSynchronizationListener(new SynchronizationMessageDialog(GuiAudioRecorder.this));
				syncManager.initialize();
			}
		}.start();
	}

	private void updateTitle() {
		AudioProject audioProject = audioProjectLifecycleManager.getActiveAudioProject();
		if (audioProject != null) {
			AudioProjectModificationObserver observer = audioProjectLifecycleManager.getActiveAudioProjectModificationObserver();
			setTitle(brandName + " - " + (observer != null && observer.isModified() ? "*" : "") + audioProject.getUniqueName());
		} else {
			setTitle(brandName);
		}
	}

	void exit() {
		getGuiSettings().setStartEmpty(audioProjectLifecycleManager.getAudioProjects().isEmpty());
		try {
			if (!audioProjectLifecycleManager.closeAllAudioProjects()) {
				return;
			}
			SyncManager.getInstance().closeAllConnections();
			getGuiSettings().store();
			AudioMixer.getInstance().store();
			dispose();
			System.exit(0);
		} catch (IOException e) {
			Dialog.message(this, "Closing audio projects", e);
		}
	}

	@Override
	public void audioProjectOpened(AudioProject audioProject) {
		audioProjectComponents.put(audioProject, new AudioProjectComponent(audioProject));
		new AudioProjectModificationObserver(audioProject).addAudioProjectModificationObserverListener(new AudioProjectModificationObserverListener() {
			@Override
			public void audioProjectModified(AudioProject audioProject) {
				updateTitle();
			}
		});
	}

	@Override
	public void audioProjectClosed(AudioProject audioProject) {
		AudioProjectComponent comp = audioProjectComponents.remove(audioProject);
		if (comp != null) {
			backgroundImageComponent.remove(comp);
			backgroundImageComponent.revalidate();
			backgroundImageComponent.repaint();
		}
		if (audioProjectComponents.isEmpty()) {
			updateTitle();
		}
	}

	@Override
	public void audioProjectActivated(AudioProject audioProject) {
		updateTitle();
		AudioProjectComponent audioProjectComponent = audioProjectComponents.get(audioProjectLifecycleManager.getActiveAudioProject());
		if (audioProjectComponent != null) {
			for (Component comp : backgroundImageComponent.getComponents()) {
				backgroundImageComponent.remove(comp);
			}
			backgroundImageComponent.add(audioProjectComponent, BorderLayout.CENTER);
			backgroundImageComponent.revalidate();
			backgroundImageComponent.repaint();
		}
	}

	AudioProjectLifecycleManager getAudioProjectLifecycleManager() {
		return audioProjectLifecycleManager;
	}

	@Override
	public void audioProjectStored(AudioProject audioProject) {
		updateTitle();
	}

	public static void main(String[] args) {
		GuiAudioRecorder guiAudioRecorder = new GuiAudioRecorder();
		if (args.length > 0) {
			File file;
			try {
				URI uri = new URI(args[0]);
				file = new File(uri);
			} catch (Exception e) {
				file = new File(args[0]);
			}
			guiAudioRecorder.getAudioProjectLifecycleManager().loadAudioProject(file);
		} else if (!getGuiSettings().isStartEmpty()) {
			guiAudioRecorder.getAudioProjectLifecycleManager().loadLastOpenedAudioProject();
		}
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/gui/mixer/GuiAudioRecorder.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.gui.mixer;

import static java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager;
import static org.audiochain.ui.gui.BackgroundImageComponent.readIcon;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;

import org.audiochain.model.AudioMixer;
import org.audiochain.model.AudioProject;
import org.audiochain.model.AudioProjectLifecycleListener;
import org.audiochain.ui.AudioProjectModificationObserver;
import org.audiochain.ui.AudioProjectModificationObserverListener;
import org.audiochain.ui.gui.BackgroundImageComponent;
import org.audiochain.ui.gui.Dialog;
import org.audiochain.ui.gui.GuiSettings;
import org.audiochain.ui.sync.SyncManager;


/**
 * This is the GUI main class.
 */
public class GuiAudioRecorder extends JFrame implements AudioProjectLifecycleListener {
	private static final long serialVersionUID = 1L;
	private static final String brandName = "AudioChain"; // ✇ is unicode decimal 9991, hex 2797
	private Map<AudioProject, AudioProjectComponent> audioProjectComponents = new LinkedHashMap<AudioProject, AudioProjectComponent>();
	private JComponent backgroundImageComponent;
	private final AudioProjectLifecycleManager audioProjectLifecycleManager;
	private final KeyEventManager keyEventManager;

	public GuiAudioRecorder() {
		audioProjectLifecycleManager = new AudioProjectLifecycleManager();
		audioProjectLifecycleManager.addAudioProjectLifecycleListener(this);
		keyEventManager = new KeyEventManager(this);
		init();
	}

	private void init() {
		final GuiSettings settings = GuiSettings.getInstance();
		settings.initializeUIManager();
		getCurrentKeyboardFocusManager().addKeyEventDispatcher(keyEventManager);
		setJMenuBar(new GuiAudioRecorderMenu(this));
		backgroundImageComponent = new BackgroundImageComponent("/org/audiochain/ui/gui/background.png");
		backgroundImageComponent.setLayout(new BorderLayout());
		setContentPane(backgroundImageComponent);
		setBounds(settings.getWindowBounds());
		updateTitle();
		setVisible(true);
		setExtendedState(settings.getWindowExtendedState());
		ImageIcon icon = readIcon("/org/audiochain/ui/gui/mixer/frameicon.png");
		if (icon != null) {
			setIconImage(icon.getImage());
		}
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				if (getExtendedState() == NORMAL) {
					settings.setWindowWidth(getWidth());
					settings.setWindowHeight(getHeight());
				}
			}

			@Override
			public void componentMoved(ComponentEvent e) {
				if (getExtendedState() == NORMAL) {
					settings.setWindowLocationX(getX());
					settings.setWindowLocationY(getY());
				}
			}
		});
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				exit();
			}
		});
		addWindowStateListener(new WindowStateListener() {
			@Override
			public void windowStateChanged(WindowEvent e) {
				settings.setWindowExtendedState(getExtendedState());
			}
		});
		final SyncManager syncManager = SyncManager.getInstance();
		syncManager.setAudioProjectLifecycleObservable(audioProjectLifecycleManager);

		new Thread() {
			@Override
			public void run() {
				setPriority(MIN_PRIORITY);
				AudioMixer.getInstance();
				syncManager.addSynchronizationListener(new SynchronizationMessageDialog(GuiAudioRecorder.this));
				syncManager.initialize();
			}
		}.start();
	}

	private void updateTitle() {
		AudioProject audioProject = audioProjectLifecycleManager.getActiveAudioProject();
		if (audioProject != null) {
			AudioProjectModificationObserver observer = audioProjectLifecycleManager.getActiveAudioProjectModificationObserver();
			setTitle(brandName + " - " + (observer != null && observer.isModified() ? "*" : "") + audioProject.getUniqueName());
		} else {
			setTitle(brandName);
		}
	}

	void exit() {
		GuiSettings.getInstance().setStartEmpty(audioProjectLifecycleManager.getAudioProjects().isEmpty());
		try {
			if (!audioProjectLifecycleManager.closeAllAudioProjects()) {
				return;
			}
			SyncManager.getInstance().closeAllConnections();
			GuiSettings.getInstance().store();
			AudioMixer.getInstance().store();
			dispose();
			System.exit(0);
		} catch (IOException e) {
			Dialog.message(this, "Closing audio projects", e);
		}
	}

	@Override
	public void audioProjectOpened(AudioProject audioProject) {
		audioProjectComponents.put(audioProject, new AudioProjectComponent(audioProject));
		new AudioProjectModificationObserver(audioProject).addAudioProjectModificationObserverListener(new AudioProjectModificationObserverListener() {
			@Override
			public void audioProjectModified(AudioProject audioProject) {
				updateTitle();
			}
		});
	}

	@Override
	public void audioProjectClosed(AudioProject audioProject) {
		AudioProjectComponent comp = audioProjectComponents.remove(audioProject);
		if (comp != null) {
			backgroundImageComponent.remove(comp);
			backgroundImageComponent.revalidate();
			backgroundImageComponent.repaint();
		}
		if (audioProjectComponents.isEmpty()) {
			updateTitle();
		}
	}

	@Override
	public void audioProjectActivated(AudioProject audioProject) {
		updateTitle();
		AudioProjectComponent audioProjectComponent = audioProjectComponents.get(audioProjectLifecycleManager.getActiveAudioProject());
		if (audioProjectComponent != null) {
			for (Component comp : backgroundImageComponent.getComponents()) {
				backgroundImageComponent.remove(comp);
			}
			backgroundImageComponent.add(audioProjectComponent, BorderLayout.CENTER);
			backgroundImageComponent.revalidate();
			backgroundImageComponent.repaint();
		}
	}

	AudioProjectLifecycleManager getAudioProjectLifecycleManager() {
		return audioProjectLifecycleManager;
	}

	@Override
	public void audioProjectStored(AudioProject audioProject) {
		updateTitle();
	}

	public static void main(String[] args) {
		GuiAudioRecorder guiAudioRecorder = new GuiAudioRecorder();
		if (args.length > 0) {
			File file;
			try {
				URI uri = new URI(args[0]);
				file = new File(uri);
			} catch (Exception e) {
				file = new File(args[0]);
			}
			guiAudioRecorder.getAudioProjectLifecycleManager().loadAudioProject(file);
		} else if (!GuiSettings.getInstance().isStartEmpty()) {
			guiAudioRecorder.getAudioProjectLifecycleManager().loadLastOpenedAudioProject();
		}
	}
}
